library session;

import 'dart:convert';

import 'package:account_kit/entities/user/user.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

import '../../serializers.dart';

part 'session.g.dart';

abstract class Session implements Built<Session, SessionBuilder> {
  Session._();

  factory Session([updates(SessionBuilder b)]) = _$Session;

  @BuiltValueField(wireName: 'token')
  String get token;

  @BuiltValueField(wireName: 'uuid')
  String get uuid;

  @BuiltValueField(wireName: 'domain')
  String get domain;

  @BuiltValueField(wireName: 'validTo')
  String get validTo;

  @BuiltValueField(wireName: 'shortCode')
  bool get shortCode;

  @BuiltValueField(wireName: 'apiKey')
  bool get apiKey;

  @BuiltValueField(wireName: 'impersonate')
  bool get impersonate;

  @BuiltValueField(wireName: 'impersonateUUID')
  String? get impersonateUUID;

  // @BuiltValueField(wireName: 'claims')
  // BuiltList<Condition> get claims;

  // @BuiltValueField(wireName: 'scopes')
  // BuiltList<Condition> get scopes;

  @BuiltValueField(wireName: 'user')
  User? get user;

  @BuiltValueField(wireName: 'createdAt')
  String? get createdAt;

  @BuiltValueField(wireName: 'updatedAt')
  String? get updatedAt;

  @BuiltValueField(wireName: 'accessAt')
  String? get accessAt;

  String toJson() {
    return json.encode(serializers.serializeWith(Session.serializer, this));
  }

  static Session? fromJson(String jsonString) {
    return serializers.deserializeWith(
        Session.serializer, json.decode(jsonString));
  }

  static BuiltList<Session> listFromJson(jsonString) {
    return BuiltList<Session>.from(json
        .decode(jsonString)
        .map((value) => serializers.deserializeWith(Session.serializer, value))
        .toList());
  }

  static Serializer<Session> get serializer => _$sessionSerializer;
}
