library user;

import 'dart:convert';

import 'package:account_kit/entities/organizationGroup/organizationGroup.dart';
import 'package:account_kit/entities/services/services.dart';
import 'package:account_kit/entities/userGroup/userGroup.dart';
import 'package:account_kit/entities/userOrganization/userOrganization.dart';
import 'package:account_kit/entities/verificationCode/verificationCode.dart';
import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

import '../../serializers.dart';

part 'user.g.dart';

abstract class User implements Built<User, UserBuilder> {
  User._();

  factory User([updates(UserBuilder b)]) = _$User;

  @BuiltValueField(wireName: 'id')
  int get id;

  @BuiltValueField(wireName: 'uuid')
  String get uuid;

  @BuiltValueField(wireName: 'firstName')
  String get firstName;

  @BuiltValueField(wireName: 'lastName')
  String get lastName;

  @BuiltValueField(wireName: 'email')
  String get email;

  @BuiltValueField(wireName: 'phone')
  String get phone;

  @BuiltValueField(wireName: 'organizationID')
  int get organizationID;

  @BuiltValueField(wireName: 'defaultAppID')
  int get defaultAppID;

  @BuiltValueField(wireName: 'defaultAppURL')
  String get defaultAppURL;

  @BuiltValueField(wireName: 'icon')
  String get icon;

  @BuiltValueField(wireName: 'active')
  bool get active;

  // @BuiltValueField(wireName: 'services')
  // Map<String, any> get services;

  @BuiltValueField(wireName: 'services')
  Services? get services;

  @BuiltValueField(wireName: 'impersonate')
  bool get impersonate;

  @BuiltValueField(wireName: 'token')
  String? get token;

  @BuiltValueField(wireName: 'pending')
  bool get pending;

  @BuiltValueField(wireName: 'relationID')
  int? get relationID;

  @BuiltValueField(wireName: 'createdAt')
  String get createdAt;

  @BuiltValueField(wireName: 'updatedAt')
  String get updatedAt;

  @BuiltValueField(wireName: 'organizations')
  BuiltList<UserOrganization> get organizations;

  @BuiltValueField(wireName: 'groups')
  BuiltList<UserGroup> get groups;

  @BuiltValueField(wireName: 'organizationGroups')
  BuiltList<OrganizationGroup> get organizationGroups;

  @BuiltValueField(wireName: 'verificationCodes')
  BuiltList<VerificationCode> get verificationCodes;

  @BuiltValueField(wireName: 'tempUUID')
  String? get tempUUID;

  String toJson() {
    return json.encode(serializers.serializeWith(User.serializer, this));
  }

  static User? fromJson(String jsonString) {
    return serializers.deserializeWith(
        User.serializer, json.decode(jsonString));
  }

  static BuiltList<User> listFromJson(jsonString) {
    return BuiltList<User>.from(json
        .decode(jsonString)
        .map((value) => serializers.deserializeWith(User.serializer, value))
        .toList());
  }

  static Serializer<User> get serializer => _$userSerializer;
}

// library user;

// import 'dart:convert';

// import 'package:account_kit/entities/services/services.dart';
// import 'package:built_collection/built_collection.dart';
// import 'package:built_value/built_value.dart';
// import 'package:built_value/serializer.dart';

// import '../../serializers.dart';

// part 'user.g.dart';

// abstract class User implements Built<User, UserBuilder> {
//   User._();

//   factory User([updates(UserBuilder b)]) = _$User;

//   @BuiltValueField(wireName: 'id')
//   int get id;
//   @BuiltValueField(wireName: 'uuid')
//   String get uuid;
//   @BuiltValueField(wireName: 'firstName')
//   String get firstName;
//   @BuiltValueField(wireName: 'lastName')
//   String get lastName;
//   @BuiltValueField(wireName: 'email')
//   String get email;
//   @BuiltValueField(wireName: 'phone')
//   String? get phone;
//   @BuiltValueField(wireName: 'organizationID')
//   int get organizationID;
//   @BuiltValueField(wireName: 'defaultAppID')
//   int get defaultAppID;
//   @BuiltValueField(wireName: 'defaultAppURL')
//   String get defaultAppURL;
//   @BuiltValueField(wireName: 'icon')
//   String get icon;
//   @BuiltValueField(wireName: 'active')
//   bool get active;
//   @BuiltValueField(wireName: 'impersonate')
//   bool get impersonate;
//   @BuiltValueField(wireName: 'token')
//   String? get token;
//   @BuiltValueField(wireName: 'pending')
//   bool get pending;
//   @BuiltValueField(wireName: 'createdAt')
//   String get createdAt;
//   @BuiltValueField(wireName: 'updatedAt')
//   String get updatedAt;
//   @BuiltValueField(wireName: 'services')
//   Services? get services;
//   String toJson() {
//     return json.encode(serializers.serializeWith(User.serializer, this));
//   }

//   static User? fromJson(jsonString) {
//     return serializers.deserializeWith(
//         User.serializer, json.decode(jsonString));
//   }

//   static Serializer<User> get serializer => _$userSerializer;

//   static BuiltList<User> listFromJson(jsonString) {
//     print(jsonString);
//     return BuiltList<User>.from(json
//         .decode(jsonString)
//         .map((value) => serializers.deserializeWith(User.serializer, value))
//         .toList());
//   }
// }
