import 'package:account_kit/helpers/regex.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/widget/apiButton.dart';
import 'package:redux/redux.dart';

class PlCodeRequest<T extends VirtualAccountState> extends StatefulWidget {
  const PlCodeRequest({
    Key? key,
  }) : super(key: key);

  @override
  _PlCodeRequestState createState() => _PlCodeRequestState<T>();
}

class _PlCodeRequestState<T extends VirtualAccountState>
    extends State<PlCodeRequest> {
  String title = "";
  String _credential = '';
  bool showApiError = true;
  TextEditingController credentialController = TextEditingController();
  final FocusNode _credentialFocus = FocusNode();

  @override
  void initState() {
    super.initState();

    setState(() {});
  }

  handleCredentialChange(value) {
    setState(() {
      _credential = value;
      showApiError = false;
    });
  }

  bool isButtonDisabled(_ViewModel vm) {
    print(_credential);

    if (isEmailMatch(_credential)) {
      return false;
    }

    if (isPhoneMatch(_credential)) {
      return false;
    }

    return true;
  }

  dynamic sendCodeRequest(_ViewModel vm) {
    print(_credential);

    if (isEmailMatch(_credential)) {
      return vm.postRequestPLCode(_credential, "");
    }

    if (isPhoneMatch(_credential)) {
      return vm.postRequestPLCode("", _credential);
    }

    return null;
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onDidChange: (vm1, vm2) {},
      onInitialBuild: (vm) {},
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return Column(
      children: [
        TextFormField(
          autofocus: true,
          enableSuggestions: false,
          autocorrect: false,
          controller: credentialController,
          keyboardAppearance: Theme.of(context).brightness,
          focusNode: _credentialFocus,
          onChanged: this.handleCredentialChange,
          decoration: InputDecoration(
            hintText: 'Email or Phone',
          ),
          onFieldSubmitted: (value) {
            FocusScope.of(context).unfocus();
          },
          keyboardType: TextInputType.emailAddress,
          validator: (value) {
            if (value!.isEmpty) {
              return 'Please enter some text';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        ApiButton<T>(
          label: "Send Code",
          disabled: isButtonDisabled(vm),
          color: Theme.of(context).colorScheme.secondary,
          apiCall: () {
            FocusScope.of(context).unfocus();
            return sendCodeRequest(vm);
          },
        ),
      ],
    );
  }
}

class _ViewModel {
  final Function(String, String) postRequestPLCode;

  _ViewModel(
    this.postRequestPLCode,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      (email, phone) => store.dispatch(PostRequestPLCode(email, phone)),
    );
  }
}
