import 'package:account_kit/entities/request/request.dart';
import 'package:account_kit/helpers/regex.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:puma_network/widget/apiButton.dart';
import 'package:puma_ui/ui/atoms/pumaButton.dart';
import 'package:redux/redux.dart';

class PlCodeSignin<T extends VirtualAccountState> extends StatefulWidget {
  const PlCodeSignin({
    Key? key,
  }) : super(key: key);

  @override
  _PlCodeSigninState createState() => _PlCodeSigninState<T>();
}

class _PlCodeSigninState<T extends VirtualAccountState>
    extends State<PlCodeSignin> {
  TextEditingController codeController = TextEditingController();
  bool preventAutofocus = false;
  final FocusNode _codeFocus = FocusNode();
  String _code = '';

  @override
  void initState() {
    super.initState();

    setState(() {});
  }

  handleCodeChange(value) {
    setState(() {
      _code = value;
    });
  }

  bool isButtonDisabled(_ViewModel vm) {
    return !isCodeMatch(_code);
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onDidChange: (vm1, vm2) {
        if (vm2.userRequest != null && !preventAutofocus) {
          preventAutofocus = true;
          _codeFocus.requestFocus();
        }
      },
      onInitialBuild: (vm) {
        _codeFocus.requestFocus();
      },
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return Column(
      children: [
        Center(
          child: Card(
            child: Padding(
              padding: const EdgeInsets.all(8.0),
              child: Wrap(
                crossAxisAlignment: WrapCrossAlignment.center,
                spacing: 4,
                children: [
                  Icon(
                    Icons.email_outlined,
                    size: 16,
                  ),
                  Text(
                    "You should have received a code by email or SMS",
                    style: Theme.of(context).textTheme.caption,
                  ),
                ],
              ),
            ),
          ),
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        TextFormField(
          controller: codeController,
          focusNode: _codeFocus,
          onChanged: this.handleCodeChange,
          keyboardAppearance: Theme.of(context).brightness,
          keyboardType: TextInputType.number,
          decoration: const InputDecoration(
            hintText: 'Enter the 6 digit code',
          ),
          validator: (value) {
            if (value != null && value.isEmpty) {
              return 'Please enter some text';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        Row(
          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
          crossAxisAlignment: CrossAxisAlignment.center,
          children: [
            PumaButton(
              title: "Cancel",
              textcolor: Colors.black,
              onPressed: () {
                vm.clearApiError();
                vm.clearSessionRequest();
                FocusScope.of(context).unfocus();
                // _emailFocus.requestFocus();
              },
            ),
            ApiButton<T>(
              label: "Submit Code",
              disabled: isButtonDisabled(vm),
              color: Theme.of(context).colorScheme.secondary,
              apiCall: () {
                FocusScope.of(context).unfocus();
                // Send code if present with 6 digits
                return vm.postPLSignin(this.codeController.text);
              },
            ),
          ],
        ),
      ],
    );
  }
}

class _ViewModel {
  final Function(String) postPLSignin;
  final Function() clearSessionRequest;
  final Request? userRequest;
  final Function clearApiError;

  _ViewModel(
    this.postPLSignin,
    this.clearSessionRequest,
    this.userRequest,
    this.clearApiError,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      (code) => store.dispatch(PostPLSignin(code)),
      () => store.dispatch(ClearSessionRequestAction()),
      store.state.accountKitState.userState.request,
      () => store.dispatch(ClearLastApiError()),
    );
  }
}
