import 'package:account_kit/helpers/regex.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:account_kit/ui/atoms/apiErrorRow.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/entities/apiError.dart';
import 'package:puma_network/widget/apiButton.dart';
import 'package:redux/redux.dart';

class PlCodeSignup<T extends VirtualAccountState> extends StatefulWidget {
  const PlCodeSignup({
    Key? key,
  }) : super(key: key);

  @override
  _PlCodeSignupState createState() => _PlCodeSignupState<T>();
}

class _PlCodeSignupState<T extends VirtualAccountState>
    extends State<PlCodeSignup> {
  bool showApiError = true;
  String _credential = '';
  String _companyToken = '';
  String _firstName = '';
  String _lastName = '';

  TextEditingController tokenController = TextEditingController();
  TextEditingController credentialController = TextEditingController();
  TextEditingController firstNameController = TextEditingController();
  TextEditingController lastNameController = TextEditingController();

  final FocusNode _tokenFocus = FocusNode();
  final FocusNode _credentialFocus = FocusNode();
  final FocusNode _firstNameFocus = FocusNode();
  final FocusNode _lastNameFocus = FocusNode();

  @override
  void initState() {
    super.initState();

    setState(() {});
  }

  bool isButtonDisabled(_ViewModel vm) {
    print(_credential);

    if (_companyToken.isEmpty) {
      return true;
    }

    if (isEmailMatch(_credential)) {
      return false;
    }

    if (isPhoneMatch(_credential)) {
      return false;
    }

    return true;
  }

  handleCredentialChange(value) {
    setState(() {
      _credential = value;
      showApiError = false;
    });
  }

  handleCompanyTokenChange(value) {
    setState(() {
      _companyToken = value;
      showApiError = false;
    });
  }

  handleFirstNameChange(value) {
    setState(() {
      _firstName = value;
      showApiError = false;
    });
  }

  handleLastNameChange(value) {
    setState(() {
      _lastName = value;
      showApiError = false;
    });
  }

  dynamic sendCodeRequest(_ViewModel vm) {
    print(_credential);

    if (isEmailMatch(_credential)) {
      return vm.postPLSignup(
        _firstName,
        _lastName,
        _credential,
        "",
        _companyToken,
      );
    }

    if (isPhoneMatch(_credential)) {
      return vm.postPLSignup(
        _firstName,
        _lastName,
        "",
        _credential,
        _companyToken,
      );
    }

    return null;
  }

  String apiErrorMessage(ApiError? apiError) {
    if (apiError == null) return "";

    if (apiError.description == "invalid token") {
      return "Invalid Company Token";
    }

    return apiError.description ?? "";
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onDidChange: (vm1, vm2) {
        setState(() {
          showApiError = true;
        });
      },
      onInitialBuild: (vm) {},
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    String apiError = apiErrorMessage(vm.apiError);

    return Column(
      children: [
        if (apiError.isNotEmpty && showApiError)
          ApiErrorRow(
            apiError: apiError,
          ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        TextFormField(
          keyboardAppearance: Theme.of(context).brightness,
          controller: tokenController,
          focusNode: _tokenFocus,
          autofocus: true,
          autocorrect: false,
          onChanged: this.handleCompanyTokenChange,
          decoration: const InputDecoration(
            hintText: 'Company Token',
          ),
          onFieldSubmitted: (value) {
            _firstNameFocus.unfocus();
            FocusScope.of(context).requestFocus(_credentialFocus);
          },
          validator: (value) {
            if (value != null && value.isEmpty) {
              return 'Please enter your company token';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        TextFormField(
          autofocus: true,
          enableSuggestions: false,
          autocorrect: false,
          controller: credentialController,
          keyboardAppearance: Theme.of(context).brightness,
          focusNode: _credentialFocus,
          onChanged: this.handleCredentialChange,
          decoration: InputDecoration(
            hintText: 'Email or Phone',
          ),
          onFieldSubmitted: (value) {
            _credentialFocus.unfocus();
            FocusScope.of(context).requestFocus(_firstNameFocus);
          },
          keyboardType: TextInputType.emailAddress,
          validator: (value) {
            if (value!.isEmpty) {
              return 'Please enter some text';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        TextFormField(
          keyboardAppearance: Theme.of(context).brightness,
          controller: firstNameController,
          focusNode: _firstNameFocus,
          autofocus: true,
          onChanged: this.handleFirstNameChange,
          decoration: const InputDecoration(
            hintText: 'First Name',
          ),
          onFieldSubmitted: (value) {
            _firstNameFocus.unfocus();
            FocusScope.of(context).requestFocus(_lastNameFocus);
          },
          validator: (value) {
            if (value != null && value.isEmpty) {
              return 'Please enter your first name';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        TextFormField(
          keyboardAppearance: Theme.of(context).brightness,
          controller: lastNameController,
          focusNode: _lastNameFocus,
          onChanged: this.handleLastNameChange,
          decoration: const InputDecoration(
            hintText: 'Last Name',
          ),
          onFieldSubmitted: (value) {
            FocusScope.of(context).unfocus();
          },
          validator: (value) {
            if (value != null && value.isEmpty) {
              return 'Please enter your last name';
            }
            return null;
          },
        ),
        Container(
          padding: const EdgeInsets.all(5.0),
        ),
        ApiButton<T>(
          label: "Sign Up",
          disabled: isButtonDisabled(vm),
          color: Theme.of(context).colorScheme.secondary,
          apiCall: () {
            FocusScope.of(context).unfocus();
            return sendCodeRequest(vm);
          },
        ),
      ],
    );
  }
}

class _ViewModel {
  final Function(String, String, String, String, String) postPLSignup;
  final ApiError? apiError;

  _ViewModel(
    this.postPLSignup,
    this.apiError,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      (firstName, lastName, email, phone, organizationToken) =>
          store.dispatch(PostPLSignup(
        firstName: firstName,
        lastName: lastName,
        email: email,
        phone: phone,
        organizationToken: organizationToken,
      )),
      store.state.networkState.lastApiError,
    );
  }
}
