import 'package:account_kit/entities/group/group.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/group/groupActions.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:redux/redux.dart';

class ImpersonateGroupTemplate<T extends VirtualAccountState>
    extends StatefulWidget {
  final Function(int)? onSelect;

  const ImpersonateGroupTemplate({
    Key? key,
    this.onSelect,
  }) : super(key: key);

  @override
  _ImpersonateGroupTemplateState createState() =>
      _ImpersonateGroupTemplateState<T>();
}

class _ImpersonateGroupTemplateState<T extends VirtualAccountState>
    extends State<ImpersonateGroupTemplate> {
  List<Group> groups = [];

  @override
  void initState() {
    super.initState();

    setState(() {});
  }

  @override
  void didUpdateWidget(ImpersonateGroupTemplate oldWidget) {
    super.didUpdateWidget(oldWidget);

    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onInitialBuild: (vm) {
        vm.getGroups();
      },
      onDidChange: (vm1, vm2) {
        setState(() {
          groups = vm2.groups ?? [];
        });
      },
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return ListView(
      children: List.generate(groups.length, (index) {
        final group = groups[index];
        return ListTile(
          onTap: () {
            vm.getGroupUsers(group.id ?? 0);
            if (widget.onSelect != null) {
              widget.onSelect!(group.id ?? 0);
            }
          },
          // onTap: () {
          // Navigator.push(
          //   context,
          //   MaterialPageRoute(
          //     builder: (context) => ImpersonateUserTemplate<T>(
          //       groupID: group.id ?? 0,
          //     ),
          //   ),
          // );
          // },
          title: Text("${group.name}"),
          trailing: Icon(
            CupertinoIcons.chevron_right,
            size: 16,
          ),
        );
      }),
    );
  }
}

class _ViewModel {
  List<Group>? groups;
  Function getGroups;
  Function(int) getGroupUsers;

  _ViewModel(
    this.groups,
    this.getGroups,
    this.getGroupUsers,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.groupState.groups?.toList(),
      () => store.dispatch(GetGroupsAction()),
      (groupID) => store.dispatch(GetGroupUsersAction(groupID: groupID)),
    );
  }
}
