import 'package:account_kit/entities/user/user.dart';
import 'package:account_kit/helpers/queryParams.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:built_collection/built_collection.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:redux/redux.dart';

class ImpersonateUserTemplate<T extends VirtualAccountState>
    extends StatefulWidget {
  final int? groupID;
  final Function(String)? onSelect;
  final bool searchEnabled;

  const ImpersonateUserTemplate({
    Key? key,
    this.groupID,
    this.onSelect,
    this.searchEnabled = false,
  }) : super(key: key);

  @override
  _ImpersonateUserTemplateState createState() =>
      _ImpersonateUserTemplateState<T>();
}

class _ImpersonateUserTemplateState<T extends VirtualAccountState>
    extends State<ImpersonateUserTemplate> {
  int? groupID;
  bool searchEnabled = false;
  TextEditingController editingController = TextEditingController();

  @override
  void initState() {
    super.initState();

    setState(() {
      groupID = widget.groupID;
      searchEnabled = widget.searchEnabled;
    });
  }

  @override
  void didUpdateWidget(ImpersonateUserTemplate oldWidget) {
    super.didUpdateWidget(oldWidget);

    setState(() {
      groupID = widget.groupID;
      searchEnabled = widget.searchEnabled;
    });
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onInitialBuild: (vm) {},
      onDidChange: (vm1, vm2) {},
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  List<User> filterSearchResults(String query, _ViewModel vm) {
    var _list = List<User>.empty(growable: true);

    if (query.isEmpty) {
      _list = vm.users;
    } else {
      vm.users.forEach((user) {
        if (user.firstName.toLowerCase().contains(query) ||
            user.lastName.toLowerCase().contains(query) ||
            user.email.toLowerCase().contains(query)) {
          _list.add(user);
        }
      });
    }

    return _list;
  }

  List<User> getUsersList(_ViewModel vm) {
    if (groupID != null && groupID! > 0) {
      return vm.groupUsers?[widget.groupID]?.toList() ?? [];
    }
    return filterSearchResults(editingController.text, vm);
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    List<User> users = getUsersList(vm);

    return Column(
      children: [
        if (searchEnabled)
          Padding(
            padding: const EdgeInsets.all(8.0),
            child: TextField(
              autofocus: true,
              onChanged: (e) {
                vm.getUsers(searchUsersQueryParams(e.toLowerCase()));
              },
              controller: editingController,
              decoration: InputDecoration(
                hintText: "Email | Name",
                prefixIcon: Icon(Icons.search),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.all(
                    Radius.circular(10.0),
                  ),
                ),
              ),
            ),
          ),
        Expanded(
          child: ListView(
            children: List.generate(
              users.length,
              (index) {
                final user = users[index];
                return ListTile(
                  onTap: () {
                    vm.impersonate(user.uuid);
                    if (widget.onSelect != null) {
                      widget.onSelect!(user.uuid);
                    }
                  },
                  title: Text("${user.firstName} ${user.lastName}"),
                  subtitle: Text("${user.email}"),
                  trailing: Icon(Icons.arrow_forward_ios, size: 16),
                );
              },
            ),
          ),
        ),
      ],
    );
  }
}

class _ViewModel {
  BuiltMap<int, BuiltList<User>>? groupUsers;
  Function(String) impersonate;
  List<User> users;
  Function(Map<String, String>) getUsers;

  _ViewModel(
    this.groupUsers,
    this.impersonate,
    this.users,
    this.getUsers,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.groupState.groupUsers,
      (uuid) => store.dispatch(ImpersonateAction({}, uuid)),
      store.state.accountKitState.userState.users.toList(),
      (queryParams) => store.dispatch(GetUsersAction(queryParams)),
    );
  }
}
