import 'package:account_kit/entities/request/request.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/ui/molecules/credentialsView.dart';
import 'package:account_kit/ui/organisms/plCodeSignin.dart';
import 'package:account_kit/ui/organisms/plCodeSignup.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:redux/redux.dart';

class SignupTemplate<T extends VirtualAccountState> extends StatefulWidget {
  final Widget? header;
  final Widget? footer;
  final Widget? illustration;
  final bool plEnabled;

  const SignupTemplate({
    Key? key,
    this.header,
    this.footer,
    this.illustration,
    this.plEnabled = true,
  }) : super(key: key);

  @override
  _SignupTemplateState createState() => _SignupTemplateState<T>();
}

class _SignupTemplateState<T extends VirtualAccountState>
    extends State<SignupTemplate> {
  @override
  void initState() {
    super.initState();

    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onInitialBuild: (_ViewModel vm) {
        vm.clearApiError();
      },
      onDidChange: (vm1, vm2) {},
      builder: (BuildContext context, _ViewModel vm) =>
          _createBody(context, vm),
    );
  }

  Widget _createBody(BuildContext context, _ViewModel vm) {
    return CredentialsView(
      header: widget.header,
      footer: widget.footer,
      illustration: widget.illustration,
      child: vm.userRequest == null ? PlCodeSignup<T>() : PlCodeSignin<T>(),
    );
  }
}

class _ViewModel {
  final Request? userRequest;
  final Function clearApiError;

  _ViewModel(
    this.userRequest,
    this.clearApiError,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.userState.request,
      () => store.dispatch(ClearLastApiError()),
    );
  }
}
