import 'package:account_kit/entities/sessionRequest/sessionRequest.dart';
import 'package:account_kit/redux/accountKitState.dart';
import 'package:account_kit/redux/user/userActions.dart';
import 'package:account_kit/ui/atoms/apiErrorRow.dart';
import 'package:account_kit/ui/molecules/signinMethodSwitch.dart';
import 'package:account_kit/ui/organisms/passwordSignin.dart';
import 'package:account_kit/ui/organisms/plCodeRequest.dart';
import 'package:account_kit/ui/organisms/plCodeSignin.dart';
import 'package:flutter/material.dart';
import 'package:flutter_redux/flutter_redux.dart';
import 'package:puma_network/entities/apiError.dart';
import 'package:puma_network/redux/network/networkAction.dart';
import 'package:redux/redux.dart';

class SigninForm<T extends VirtualAccountState> extends StatefulWidget {
  // final void Function(String, String) onSubmit;
  final bool plEnabled;

  SigninForm({
    this.plEnabled = false,
  });

  @override
  _SigninFormStateWidget createState() => _SigninFormStateWidget<T>();
}

/// This is the private State class that goes with SigninForm.
class _SigninFormStateWidget<T extends VirtualAccountState>
    extends State<SigninForm> {
  final _formKey = GlobalKey<FormState>();

  final FocusNode _codeFocus = FocusNode();
  bool preventAutofocus = false;
  bool plEnabled = false;
  bool showApiError = true;
  // Todo: Get Env default PLEnabled

  @override
  void initState() {
    super.initState();
    plEnabled = widget.plEnabled;
  }

  togglePlEnabled(_ViewModel vm) {
    if (plEnabled) {
      print("Dispatching clearSessionRequest");
      vm.clearSessionRequest();
    }
    setState(() {
      plEnabled = !plEnabled;
    });
  }

  String apiErrorMessage(ApiError? apiError) {
    if (apiError == null) return "";

    return apiError.description ?? "";
  }

  @override
  Widget build(BuildContext context) {
    return StoreConnector<T, _ViewModel>(
      converter: (Store<T> store) => _ViewModel.create(store),
      onDidChange: (vm1, vm2) {
        if (vm2.sessionRequest != null && !preventAutofocus) {
          preventAutofocus = true;
          _codeFocus.requestFocus();
        }
        setState(() {
          showApiError = true;
        });
      },
      builder: (BuildContext context, _ViewModel vm) {
        String apiError = apiErrorMessage(vm.apiError);

        return Form(
          key: _formKey,
          child: Wrap(
            runSpacing: 10.0,
            children: <Widget>[
              if (plEnabled && vm.sessionRequest == null) PlCodeRequest<T>(),
              if (!plEnabled && vm.sessionRequest == null) PasswordSignin<T>(),
              if (plEnabled && vm.sessionRequest != null) PlCodeSignin<T>(),
              if (apiError.isNotEmpty && showApiError)
                ApiErrorRow(
                  apiError: apiError,
                ),
              SigninMethodSwitch(
                plEnabled: plEnabled,
                toggle: () {
                  vm.clearApiError();
                  togglePlEnabled(vm);
                },
              ),
            ],
          ),
        );
      },
    );
  }
}

class _ViewModel {
  final bool signinLoading = false;
  final SessionRequest? sessionRequest;
  final Function() clearSessionRequest;
  final ApiError? apiError;
  final Function clearApiError;

  _ViewModel(
    this.sessionRequest,
    this.clearSessionRequest,
    this.apiError,
    this.clearApiError,
  );

  factory _ViewModel.create(Store<VirtualAccountState> store) {
    return _ViewModel(
      store.state.accountKitState.userState.sessionRequest,
      () => store.dispatch(ClearSessionRequestAction()),
      store.state.networkState.lastApiError,
      () => store.dispatch(ClearLastApiError()),
    );
  }
}
